@@ -1,20 +1,26 @@
 package WebService::Mailgun;
+
+
 use 5.008001;
 use strict;
 use warnings;
 
 use Furl;
-use JSON::XS;
+use JSON;
 use URI;
 use Try::Tiny;
 use Carp;
+use HTTP::Request::Common;
+use File::Temp;
 
-our $VERSION = "0.7.0";
+
+our $VERSION = "0.15.1";
 our $API_BASE = 'api.mailgun.net/v3';
+our $API_BASE_EU = 'api.eu.mailgun.net/v3';
 
 use Class::Accessor::Lite (
     new => 1,
-    rw  => [qw(api_key domain RaiseError)],
+    rw  => [qw(api_key domain RaiseError region)],
     ro  => [qw(error error_status)],
 );
 
@@ -57,8 +63,10 @@ sub recursive {
         my $res = $self->client->get($api_uri->as_string);
         my $json = $self->decode_response($res);
         unless($json && scalar @{$json->{$key}}) {
-            $previous = URI->new($json->{paging}->{previous});
-            $previous->userinfo('api:'.$self->api_key);
+            try {
+                $previous = URI->new($json->{paging}->{previous});
+                $previous->userinfo('api:'.$self->api_key);
+            } catch {};
             last;
         }
         push @result, @{$json->{$key}};
@@ -77,27 +85,112 @@ sub client {
     );
 }
 
+sub api_base {
+    my ($self) = @_;
+
+    if ($self->region) {
+        if (lc($self->region) eq "eu") {
+            return $API_BASE_EU;
+        } elsif (lc($self->region) ne "us") {
+            die "unsupported region '" . $self->region ."'";
+        }
+    }
+    return $API_BASE;
+}
+
 sub api_url {
     my ($self, $method) = @_;
 
     sprintf 'https://api:%s@%s/%s',
-        $self->api_key, $API_BASE, $method;
+        $self->api_key, $self->api_base, $method;
 }
 
 sub domain_api_url {
     my ($self, $method) = @_;
 
     sprintf 'https://api:%s@%s/%s/%s',
-        $self->api_key, $API_BASE, $self->domain, $method;
+        $self->api_key, $self->api_base, $self->domain, $method;
 }
 
 sub message {
     my ($self, $args) = @_;
 
-    my $res = $self->client->post($self->domain_api_url('messages'), [], $args);
+    my @content;
+    if (ref($args) eq 'HASH') {
+        @content = %$args;
+    }
+    elsif (ref($args) eq 'ARRAY') {
+        @content = @$args;
+    }
+    else {
+        die 'unsupport argument. message() need HashRef or ArrayRef.';    
+	}
+
+    my $req = POST $self->domain_api_url('messages'), Content_type => 'form-data', Content => \@content;
+
+    my $res = $self->client->request($req);
     $self->decode_response($res);
 }
 
+
+
+sub mime { 
+		
+    my ($self, $args) = @_;
+
+    if (ref($args) eq 'HASH') {
+        # Well, good!
+    }
+    else {
+        die 'unsupport argument. mime() needs a hash ref.';
+    }
+	
+	
+	my $tmp = undef; 
+
+	if(exists($args->{message})){
+		$tmp = File::Temp->new();
+		
+		if(ref $args->{message} eq 'SCALAR'){ 
+			# save from a ref
+			print $tmp ${$args->{message}};
+			seek $tmp, 0, 0;
+		}
+		else {
+			# Save from a string
+			print $tmp $args->{message};
+			seek $tmp, 0, 0;
+		}
+		$args->{message} = [$tmp->filename];
+	}
+	elsif(exists($args->{file})){ 
+		if(-f $args->{file}){ 
+			$args->{message} =  [$args->{file}];
+			delete $args->{file}; 
+		}
+		else { 
+			die "cannot find file, " . $args->{file}; 
+		}
+	}
+	
+	# Put it back together: 
+	my @content = %$args;
+
+    my $req = POST $self->domain_api_url('messages.mime'), 
+		Content_Type => 'form-data', 
+		Content => \@content;
+		
+    my $res = $self->client->request($req);
+	
+	undef $tmp; 
+	
+    $self->decode_response($res);
+
+}
+
+
+
+
 sub lists {
     my $self = shift;
 
@@ -198,6 +291,41 @@ sub get_message_from_event {
     $self->decode_response($res);
 }
 
+sub delete_templates {
+    my ($self) = @_;
+
+    my $res = $self->client->delete($self->domain_api_url("templates"));
+    $self->decode_response($res);
+}
+
+sub delete_template {
+    my ($self, $name) = @_;
+
+    my $res = $self->client->delete(
+        $self->domain_api_url("templates/$name"));
+    $self->decode_response($res);
+}
+
+sub add_template {
+    my ($self, $args) = @_;
+
+    my @content;
+    if (ref($args) eq 'HASH') {
+        @content = %$args;
+    }
+    elsif (ref($args) eq 'ARRAY') {
+        @content = @$args;
+    }
+    else {
+        die 'unsupport argument. add_template() need HashRef or ArrayRef.';
+    }
+
+    my $req = POST $self->domain_api_url('templates'), Content_type => 'form-data', Content => \@content;
+
+    my $res = $self->client->request($req);
+    $self->decode_response($res);
+}
+
 1;
 __END__
 
@@ -230,7 +358,7 @@ WebService::Mailgun is API client for Mailgun (L<https://mailgun.com/>).
 
 =head1 METHOD
 
-=head2 new(api_key => $api_key, domain => $domain, RaiseError => 0|1)
+=head2 new(api_key => $api_key, domain => $domain, region => "us"|"eu", RaiseError => 0|1)
 
 Create mailgun object.
 
@@ -238,6 +366,10 @@ Create mailgun object.
 
 The RaiseError attribute can be used to force errors to raise exceptions rather than simply return error codes in the normal way. It is "off" by default.
 
+=head3 region (default: "us")
+
+The region attribute determines what region the domain belongs to, either US or EU. Default is US.
+
 =head2 error
 
 return recent error message.
@@ -258,7 +390,44 @@ Send email message.
         text    => 'text',
     });
 
-L<https://documentation.mailgun.com/api-sending.html#sending>
+L<https://documentation.mailgun.com/en/latest/api-sending.html#sending>
+
+=head2 mime($args)
+
+Send a MIME message you build yourself, usually by using a library to create that MIME message. 
+The C<to> parameter needs to be passed as one of the arguments. 
+Either the C<file> or C<message> parameter will also need to be passed. 
+
+The C<file> parameter should contain the path to the filename that holds the MIME message. 
+The C<message> parameter should contain either a string or a reference to a string that holds the MIME message: 
+
+    # send MIME message via a filename: 
+    my $res = $mailgun->message({
+    	to      => 'bar@example.com',
+		file    => '/path/to/filename.mime',	
+    });
+
+    # send MIME message via a string:
+	use MIME::Entity; 
+	my $str = MIME::Entity->build(
+		From    => 'foo@example.com',
+        To      => 'bar@example.com',
+        Subject => "Subject",
+        Data    => 'Messag4')->as_string;
+	
+    my $res = $mailgun->message({
+    	to       => 'bar@example.com',
+		message  => $str,
+    });
+
+    # or send MIME message via a string ref:	
+    my $res = $mailgun->message({
+    	to       => 'bar@example.com',
+		message  => \$str,
+    });
+
+L<https://documentation.mailgun.com/en/latest/api-sending.html#sending>
+
 
 =head2 lists()
 
@@ -268,7 +437,7 @@ Get list of mailing lists.
     my $lists = $mailgun->lists();
     # => ArrayRef of mailing list object.
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 add_list($args)
 
@@ -282,7 +451,7 @@ Add mailing list.
         access_level => 'members',   # readonly(default), members, everyone
     });
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 list($address)
 
@@ -291,7 +460,7 @@ Get detail for mailing list.
     # get mailing list detail
     my $data = $mailgun->list('ml@exmaple.com');
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 update_list($address, $args)
 
@@ -305,7 +474,7 @@ Update mailing list detail.
         access_level => 'members',   # readonly(default), members, everyone
     });
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 delete_list($address)
 
@@ -314,7 +483,7 @@ Delete mailing list.
     # delete mailing list
     my $res = $mailgun->delete_list('ml@example.com');
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 list_members($address)
 
@@ -323,7 +492,7 @@ Get members for mailing list.
     # get members
     my $res = $mailgun->list_members('ml@example.com');
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 add_list_member($address, $args)
 
@@ -338,13 +507,13 @@ Add member for mailing list.
         upsert     => 'no',            # no (default). if yes, update exists member
     });
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 add_list_members($address, $args)
 
 Adds multiple members for mailing list.
 
-    use JSON::XS; # auto export 'encode_json'
+    use JSON; # auto export 'encode_json'
 
     # add members
     my $res = $mailgun->add_list_members('ml@example.com' => {
@@ -361,7 +530,7 @@ Adds multiple members for mailing list.
         members => encode_json [qw/user1@example.com user2@example.com/],
     });
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 list_member($address, $member_address)
 
@@ -370,7 +539,7 @@ Get member detail.
     # update member
     my $res = $mailgun->list_member('ml@example.com', 'user@example.com');
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 update_list_member($address, $member_address, $args)
 
@@ -384,7 +553,7 @@ Update member detail.
         subscribed => 'yes',           # yes(default) or no
     });
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 delete_list_members($address, $member_address)
 
@@ -393,7 +562,7 @@ Delete member for mailing list.
     # delete member
     my $res = $mailgun->delete_list_member('ml@example.com' => 'user@example.com');
 
-L<https://documentation.mailgun.com/api-mailinglists.html#mailing-lists>
+L<https://documentation.mailgun.com/en/latest/api-mailinglists.html#mailing-lists>
 
 =head2 event($args)
 
@@ -402,7 +571,7 @@ Get event data.
     # get event data
     my ($events, $purl) = $mailgun->event({ event => 'stored', limit => 50 });
 
-L<Events|https://documentation.mailgun.com/api-events.html>
+L<Events|https://documentation.mailgun.com/en/latest/api-events.html>
 
 =head2 get_message_from_event($event)
 
@@ -412,7 +581,39 @@ Get stored message.
     my ($events, $purl) = $mailgun->event({ event => 'stored' });
     my $msg = $mailgun->get_message_from_event($events->[0]);
 
-L<Stored Message|https://documentation.mailgun.com/api-sending.html#retrieving-stored-messages>
+L<Stored Message|https://documentation.mailgun.com/en/latest/api-sending.html#retrieving-stored-messages>
+
+=head2 add_template($args)
+
+Add a template
+
+    # add template
+    my $res = $mailgun->add_template({
+        name        => 'welcome',     # Template name
+        template    => 'Hello!',      # Template data
+        engine      => 'handlebars',  # Template engine (optional)
+        description => 'xyz',         # Description of template (optional)
+        tag         => '2.0' ,        # Version tag (optional)
+        comment     => 'Test'         # Version comment (optional)
+    });
+
+L<https://documentation.mailgun.com/en/latest/api-templates.html#templates>
+
+=head2 delete_templates()
+
+Delete all templates
+
+    my $res = $mailgun->delete_templates();
+
+L<https://documentation.mailgun.com/en/latest/api-templates.html#templates>
+
+=head2 delete_template($name)
+
+Delete a template
+
+    my $res = $mailgun->delete_template($name);
+
+L<https://documentation.mailgun.com/en/latest/api-templates.html#templates>
 
 =head1 Event Pooling
 
@@ -424,7 +625,9 @@ event method return previous url. it can use for fetch event.
     $events = $mailgun->event($purl);
     // ...
 
-L<Event Polling|https://documentation.mailgun.com/api-events.html#event-polling>    
+L<Event Polling|https://documentation.mailgun.com/en/latest/api-events.html#event-polling>    
+
+
 
 =head1 TODO
 
@@ -432,25 +635,27 @@ this API not implement yet.
 
 =over
 
-=item * L<Domains|https://documentation.mailgun.com/api-domains.html>
+=item * L<Domains|https://documentation.mailgun.com/en/latest/api-domains.html>
 
-=item * L<Stats|https://documentation.mailgun.com/api-stats.html>
+=item * L<Stats|https://documentation.mailgun.com/en/latest/api-stats.html>
 
-=item * L<Tags|https://documentation.mailgun.com/api-tags.html>
+=item * L<Tags|https://documentation.mailgun.com/en/latest/api-tags.html>
 
-=item * L<Suppressions|https://documentation.mailgun.com/api-suppressions.html>
+=item * L<Suppressions|https://documentation.mailgun.com/en/latest/api-suppressions.html>
 
-=item * L<Routes|https://documentation.mailgun.com/api-routes.html>
+=item * L<Routes|https://documentation.mailgun.com/en/latest/api-routes.html>
 
-=item * L<Webhooks|https://documentation.mailgun.com/api-webhooks.html>
+=item * L<Webhooks|https://documentation.mailgun.com/en/latest/api-webhooks.html>
 
-=item * L<Email Validation|https://documentation.mailgun.com/api-email-validation.html>
+=item * L<Email Validation|https://documentation.mailgun.com/en/latest/api-email-validation.html>
+
+=item * L<Templates|https://documentation.mailgun.com/en/latest/api-templates.html> (partial)
 
 =back
 
 =head1 SEE ALSO
 
-L<WWW::Mailgun>, L<https://documentation.mailgun.com/>
+L<WWW::Mailgun>, L<https://documentation.mailgun.com/en/latest/>
 
 =head1 LICENSE
 
